@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache.h 4582 2010-02-21 22:00:09Z phk $
+ * $Id: cache.h 4682 2010-04-19 11:38:52Z tfheen $
  */
 
 /*
@@ -95,6 +95,15 @@ struct lock { void *priv; };		// Opaque
 #define DIGEST_LEN		32
 
 
+/*--------------------------------------------------------------------
+ * Pointer aligment magic
+ */
+
+#define PALGN		(sizeof(void *) - 1)
+#define PAOK(p)		(((uintptr_t)(p) & PALGN) == 0)
+#define PRNDDN(p)	((uintptr_t)(p) & ~PALGN)
+#define PRNDUP(p)	(((uintptr_t)(p) + PALGN) & ~PALGN)
+
 /*--------------------------------------------------------------------*/
 
 typedef struct {
@@ -135,6 +144,7 @@ enum httpwhence {
 	HTTP_Obj = 3
 };
 
+/* NB: remember to update http_Copy() if you add fields */
 struct http {
 	unsigned		magic;
 #define HTTP_MAGIC		0x6428b5c9
@@ -514,6 +524,7 @@ void Fetch_Init(void);
 
 /* cache_http.c */
 unsigned HTTP_estimate(unsigned nhttp);
+void HTTP_Copy(struct http *to, const struct http * const fm);
 struct http *HTTP_create(void *p, unsigned nhttp);
 const char *http_StatusMessage(unsigned);
 unsigned http_EstimateWS(const struct http *fm, unsigned how, unsigned *nhd);
