@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,12 +26,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_acceptor.c 3996 2009-03-23 14:33:56Z tfheen $
- *
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_acceptor.c 4605 2010-03-09 11:27:08Z phk $")
+
 #include <stdio.h>
 #include <errno.h>
 #include <poll.h>
@@ -47,32 +48,56 @@
 #include "cli_priv.h"
 #include "shmlog.h"
 #include "cache.h"
-#include "cache_acceptor.h"
+#include "cache_waiter.h"
 
-static struct acceptor * const vca_acceptors[] = {
+static struct waiter * const vca_waiters[] = {
 #if defined(HAVE_KQUEUE)
-	&acceptor_kqueue,
+	&waiter_kqueue,
 #endif
 #if defined(HAVE_EPOLL_CTL)
-	&acceptor_epoll,
+	&waiter_epoll,
 #endif
 #if defined(HAVE_PORT_CREATE)
-	&acceptor_ports,
+	&waiter_ports,
 #endif
-	&acceptor_poll,
+	&waiter_poll,
 	NULL,
 };
 
-static struct acceptor const *vca_act;
+static struct waiter const *vca_act;
 
-static pthread_t	vca_thread_acct;
+pthread_t		VCA_thread;
 static struct timeval	tv_sndtimeo;
 static struct timeval	tv_rcvtimeo;
-static struct linger	linger;
+
+/*--------------------------------------------------------------------
+ * Report waiter name to panics
+ */
+
+const char *
+VCA_waiter_name(void)
+{
+
+	if (vca_act != NULL)
+		return (vca_act->name);
+	else
+		return ("no_waiter");
+}
+
+
+/*--------------------------------------------------------------------
+ * We want to get out of any kind of touble-hit TCP connections as fast
+ * as absolutely possible, so we set them LINGER enabled with zero timeout,
+ * so that even if there are outstanding write data on the socket, a close(2)
+ * will return immediately.
+ */
+static const struct linger linger = {
+	.l_onoff	=	0,
+};
 
 static unsigned char	need_sndtimeo, need_rcvtimeo, need_linger, need_test;
 
-int vca_pipes[2];
+int vca_pipes[2] = { -1, -1 };
 
 static void
 sock_test(int fd)
@@ -80,39 +105,64 @@ sock_test(int fd)
 	struct linger lin;
 	struct timeval tv;
 	socklen_t l;
+	int i;
 
 	l = sizeof lin;
-	AZ(getsockopt(fd, SOL_SOCKET, SO_LINGER, &lin, &l));
+	i = getsockopt(fd, SOL_SOCKET, SO_LINGER, &lin, &l);
+	if (i) {
+		TCP_Assert(i);
+		return;
+	}
 	assert(l == sizeof lin);
 	if (memcmp(&lin, &linger, l))
 		need_linger = 1;
 
 #ifdef SO_SNDTIMEO_WORKS
 	l = sizeof tv;
-	AZ(getsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, &tv, &l));
+	i = getsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, &tv, &l);
+	if (i) {
+		TCP_Assert(i);
+		return;
+	}
 	assert(l == sizeof tv);
 	if (memcmp(&tv, &tv_sndtimeo, l))
 		need_sndtimeo = 1;
+#else
+	(void)tv;
+	(void)tv_sndtimeo;
+	(void)need_sndtimeo;
 #endif
 
 #ifdef SO_RCVTIMEO_WORKS
 	l = sizeof tv;
-	AZ(getsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &tv, &l));
+	i = getsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &tv, &l);
+	if (i) {
+		TCP_Assert(i);
+		return;
+	}
 	assert(l == sizeof tv);
 	if (memcmp(&tv, &tv_rcvtimeo, l))
 		need_rcvtimeo = 1;
+#else
+	(void)tv;
+	(void)tv_rcvtimeo;
+	(void)need_rcvtimeo;
 #endif
 
 	need_test = 0;
 }
 
+/*--------------------------------------------------------------------
+ * Called once the workerthread gets hold of the session, to do setup
+ * setup overhead, we don't want to bother the acceptor thread with.
+ */
+
 void
 VCA_Prep(struct sess *sp)
 {
 	char addr[TCP_ADDRBUFSIZE];
 	char port[TCP_PORTBUFSIZE];
 
-
 	TCP_name(sp->sockaddr, sp->sockaddrlen,
 	    addr, sizeof addr, port, sizeof port);
 	sp->addr = WS_Dup(sp->ws, addr);
@@ -131,16 +181,16 @@ VCA_Prep(struct sess *sp)
 	if (need_test)
 		sock_test(sp->fd);
 	if (need_linger)
-		AZ(setsockopt(sp->fd, SOL_SOCKET, SO_LINGER,
+		TCP_Assert(setsockopt(sp->fd, SOL_SOCKET, SO_LINGER,
 		    &linger, sizeof linger));
 #ifdef SO_SNDTIMEO_WORKS
 	if (need_sndtimeo)
-		AZ(setsockopt(sp->fd, SOL_SOCKET, SO_SNDTIMEO,
+		TCP_Assert(setsockopt(sp->fd, SOL_SOCKET, SO_SNDTIMEO,
 		    &tv_sndtimeo, sizeof tv_sndtimeo));
 #endif
 #ifdef SO_RCVTIMEO_WORKS
 	if (need_rcvtimeo)
-		AZ(setsockopt(sp->fd, SOL_SOCKET, SO_RCVTIMEO,
+		TCP_Assert(setsockopt(sp->fd, SOL_SOCKET, SO_RCVTIMEO,
 		    &tv_rcvtimeo, sizeof tv_rcvtimeo));
 #endif
 }
@@ -158,7 +208,7 @@ vca_acct(void *arg)
 	struct pollfd *pfd;
 	struct listen_sock *ls;
 	unsigned u;
-	double now;
+	double t0, now, pace;
 
 	THR_SetName("cache-acceptor");
 	(void)arg;
@@ -178,6 +228,8 @@ vca_acct(void *arg)
 	}
 
 	need_test = 1;
+	pace = 0;
+	t0 = TIM_real();
 	while (1) {
 #ifdef SO_SNDTIMEO_WORKS
 		if (params->send_timeout != tv_sndtimeo.tv_sec) {
@@ -205,8 +257,16 @@ vca_acct(void *arg)
 			}
 		}
 #endif
+		/* Bound the pacing delay by parameter */
+		if (pace > params->acceptor_sleep_max)
+			pace = params->acceptor_sleep_max;
+		if (pace < params->acceptor_sleep_incr)
+			pace = 0.0;
+		if (pace > 0.0)
+			TIM_sleep(pace);
 		i = poll(pfd, heritage.nsocks, 1000);
 		now = TIM_real();
+		VSL_stats->uptime = (uint64_t)(now - t0);
 		u = 0;
 		VTAILQ_FOREACH(ls, &heritage.socks, list) {
 			if (ls->sock < 0)
@@ -218,6 +278,7 @@ vca_acct(void *arg)
 			addr = (void*)&addr_s;
 			i = accept(ls->sock, addr, &l);
 			if (i < 0) {
+				VSL_stats->accept_fail++;
 				switch (errno) {
 				case EAGAIN:
 				case ECONNABORTED:
@@ -226,30 +287,43 @@ vca_acct(void *arg)
 					VSL(SLT_Debug, ls->sock,
 					    "Too many open files "
 					    "when accept(2)ing. Sleeping.");
-					TIM_sleep(
-					    params->accept_fd_holdoff * 0.001);
+					pace += params->acceptor_sleep_incr;
 					break;
 				default:
 					VSL(SLT_Debug, ls->sock,
 					    "Accept failed: %s",
 					    strerror(errno));
-					/* XXX: stats ? */
+					pace += params->acceptor_sleep_incr;
 					break;
 				}
 				continue;
 			}
-			sp = SES_New(addr, l);
-			XXXAN(sp);
-
+			sp = SES_New();
+			if (sp == NULL) {
+				AZ(close(i));
+				VSL_stats->client_drop++;
+				pace += params->acceptor_sleep_incr;
+				continue;
+			}
 			sp->fd = i;
 			sp->id = i;
 			sp->t_open = now;
+			sp->t_end = now;
 			sp->mylsock = ls;
+			assert(l < sp->sockaddrlen);
+			memcpy(sp->sockaddr, addr, l);
+			sp->sockaddrlen = l;
 
 			sp->step = STP_FIRST;
-			WRK_QueueSession(sp);
+			if (WRK_QueueSession(sp)) {
+				VSL_stats->client_drop++;
+				pace += params->acceptor_sleep_incr;
+			} else {
+				pace *= params->acceptor_sleep_decay;
+			}
 		}
 	}
+	NEEDLESS_RETURN(NULL);
 }
 
 /*--------------------------------------------------------------------*/
@@ -269,7 +343,8 @@ vca_handover(struct sess *sp, int status)
 		break;
 	case 1:
 		sp->step = STP_START;
-		WRK_QueueSession(sp);
+		if (WRK_QueueSession(sp))
+			VSL_stats->client_drop_late++;
 		break;
 	default:
 		INCOMPL();
@@ -303,14 +378,14 @@ vca_return_session(struct sess *sp)
 	 * Set nonblocking in the worker-thread, before passing to the
 	 * acceptor thread, to reduce syscall density of the latter.
 	 */
-	TCP_nonblocking(sp->fd);
-	if (vca_act->pass == NULL)
+	if (TCP_nonblocking(sp->fd))
+		vca_close_session(sp, "remote closed");
+	else if (vca_act->pass == NULL)
 		assert(sizeof sp == write(vca_pipes[1], &sp, sizeof sp));
 	else
 		vca_act->pass(sp);
 }
 
-
 /*--------------------------------------------------------------------*/
 
 static void
@@ -322,7 +397,7 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 	(void)priv;
 
 	if (vca_act == NULL)
-		vca_act = vca_acceptors[0];
+		vca_act = vca_waiters[0];
 
 	AN(vca_act);
 	AN(vca_act->name);
@@ -330,12 +405,37 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 	if (vca_act->pass == NULL)
 		AZ(pipe(vca_pipes));
 	vca_act->init();
-	AZ(pthread_create(&vca_thread_acct, NULL, vca_acct, NULL));
+	AZ(pthread_create(&VCA_thread, NULL, vca_acct, NULL));
 	VSL(SLT_Debug, 0, "Acceptor is %s", vca_act->name);
 }
 
+/*--------------------------------------------------------------------*/
+
+static void
+ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
+{
+	struct listen_sock *ls;
+	char h[32], p[32];
+
+	(void)cli;
+	(void)av;
+	(void)priv;
+	VTAILQ_FOREACH(ls, &heritage.socks, list) {
+		if (ls->sock < 0)
+			continue;
+		TCP_myname(ls->sock, h, sizeof h, p, sizeof p);
+		cli_out(cli, "%s %s\n", h, p);
+	}
+}
+
+/*--------------------------------------------------------------------*/
+
 static struct cli_proto vca_cmds[] = {
-	{ CLI_SERVER_START,	ccf_start },
+	{ CLI_SERVER_START,	"i", ccf_start },
+	{ "debug.listen_address",
+	    "debug.listen_address",
+	    "Report the actual listen address\n", 0, 0,
+	    "d", ccf_listen_address, NULL },
 	{ NULL }
 };
 
@@ -343,14 +443,31 @@ void
 VCA_Init(void)
 {
 
-	CLI_AddFuncs(MASTER_CLI, vca_cmds);
+	CLI_AddFuncs(vca_cmds);
 }
 
 void
-VCA_tweak_acceptor(struct cli *cli, const char *arg)
+VCA_Shutdown(void)
+{
+	struct listen_sock *ls;
+	int i;
+
+	VTAILQ_FOREACH(ls, &heritage.socks, list) {
+		if (ls->sock < 0)
+			continue;
+		i = ls->sock;
+		ls->sock = -1;
+		(void)close(i);
+	}
+}
+
+void
+VCA_tweak_waiter(struct cli *cli, const char *arg)
 {
 	int i;
 
+	 ASSERT_MGT();
+
 	if (arg == NULL) {
 		if (vca_act == NULL)
 			cli_out(cli, "default");
@@ -358,9 +475,9 @@ VCA_tweak_acceptor(struct cli *cli, const char *arg)
 			cli_out(cli, "%s", vca_act->name);
 
 		cli_out(cli, " (");
-		for (i = 0; vca_acceptors[i] != NULL; i++)
+		for (i = 0; vca_waiters[i] != NULL; i++)
 			cli_out(cli, "%s%s", i == 0 ? "" : ", ",
-			    vca_acceptors[i]->name);
+			    vca_waiters[i]->name);
 		cli_out(cli, ")");
 		return;
 	}
@@ -368,12 +485,12 @@ VCA_tweak_acceptor(struct cli *cli, const char *arg)
 		vca_act = NULL;
 		return;
 	}
-	for (i = 0; vca_acceptors[i]; i++) {
-		if (!strcmp(arg, vca_acceptors[i]->name)) {
-			vca_act = vca_acceptors[i];
+	for (i = 0; vca_waiters[i]; i++) {
+		if (!strcmp(arg, vca_waiters[i]->name)) {
+			vca_act = vca_waiters[i];
 			return;
 		}
 	}
-	cli_out(cli, "Unknown acceptor");
+	cli_out(cli, "Unknown waiter");
 	cli_result(cli, CLIS_PARAM);
 }
