@@ -41,7 +41,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_esi.c 4557 2010-02-12 11:58:43Z phk $")
+SVNID("$Id: cache_esi.c 4678 2010-04-19 11:16:21Z tfheen $")
 
 #include <stdio.h>
 #include <unistd.h>
@@ -896,7 +896,7 @@ ESI_Deliver(struct sess *sp)
 			http_save = *sp->http;
 
 		/* Reset request to status before we started messing with it */
-		*sp->http = *sp->http0;
+		HTTP_Copy(sp->http, sp->http0);
 
 		/* Take a workspace snapshot */
 		ws_wm = WS_Snapshot(sp->ws);
