@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_http.c 4603 2010-03-03 09:12:10Z phk $")
+SVNID("$Id: cache_http.c 4683 2010-04-19 11:44:19Z tfheen $")
 
 #include <errno.h>
 #include <unistd.h>
@@ -679,7 +679,7 @@ http_EstimateWS(const struct http *fm, unsigned how, unsigned *nhd)
 			continue;
 #include "http_headers.h"
 #undef HTTPH
-		l += Tlen(fm->hd[u]) + 1;
+		l += PRNDUP(Tlen(fm->hd[u]) + 1);
 		(*nhd)++;
 		// fm->hdf[u] |= HDF_COPY;
 	}
@@ -894,6 +894,22 @@ http_Unset(struct http *hp, const char *hdr)
 
 /*--------------------------------------------------------------------*/
 
+void
+HTTP_Copy(struct http *to, const struct http * const fm)
+{
+
+	to->conds = fm->conds;
+	to->logtag = fm->logtag;
+	to->status = fm->status;
+	to->protover = fm->protover;
+	to->nhd = fm->nhd;
+	assert(fm->nhd <= to->shd);
+	memcpy(to->hd, fm->hd, fm->nhd * sizeof *to->hd);
+	memcpy(to->hdf, fm->hdf, fm->nhd * sizeof *to->hdf);
+}
+
+/*--------------------------------------------------------------------*/
+
 unsigned
 http_Write(struct worker *w, const struct http *hp, int resp)
 {
