@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_vrt.c 4538 2010-02-08 18:09:49Z phk $")
+SVNID("$Id: cache_vrt.c 4764 2010-05-05 08:02:47Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -115,6 +115,7 @@ vrt_selecthttp(const struct sess *sp, enum gethdr_e where)
 		hp = sp->wrk->resp;
 		break;
 	case HDR_OBJ:
+		CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
 		hp = sp->obj->http;
 		break;
 	default:
@@ -867,7 +868,7 @@ void
 VRT_Rollback(struct sess *sp)
 {
 
-	*sp->http = *sp->http0;
+	HTTP_Copy(sp->http, sp->http0);
 	WS_Reset(sp->ws, sp->ws_req);
 }
 
@@ -937,7 +938,7 @@ VRT_synth_page(struct sess *sp, unsigned flags, const char *str, ...)
 /*--------------------------------------------------------------------*/
 
 void
-VRT_purge(struct sess *sp, char *cmds, ...)
+VRT_ban(struct sess *sp, char *cmds, ...)
 {
 	char *a1, *a2, *a3;
 	va_list ap;
@@ -972,7 +973,7 @@ VRT_purge(struct sess *sp, char *cmds, ...)
 /*--------------------------------------------------------------------*/
 
 void
-VRT_purge_string(struct sess *sp, const char *str, ...)
+VRT_ban_string(struct sess *sp, const char *str, ...)
 {
 	char *p, *a1, *a2, *a3;
 	char **av;
@@ -1023,6 +1024,19 @@ VRT_purge_string(struct sess *sp, const char *str, ...)
 	FreeArgv(av);
 }
 
+/*--------------------------------------------------------------------
+ * "real" purges
+ */
+
+void
+VRT_purge(struct sess *sp, double ttl, double grace)
+{
+	if (sp->cur_method == VCL_MET_HIT)
+		HSH_Purge(sp, sp->obj->objcore->objhead, ttl, grace);
+	else if (sp->cur_method == VCL_MET_MISS)
+		HSH_Purge(sp, sp->objcore->objhead, ttl, grace);
+}
+
 /*--------------------------------------------------------------------
  * Simple stuff
  */
