@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: libvarnish.h 4003 2009-03-27 12:13:31Z tfheen $
+ * $Id: libvarnish.h 4572 2010-02-17 13:12:07Z phk $
  */
 
 #include <errno.h>
@@ -42,29 +42,34 @@ struct vsb;
 /* from libvarnish/argv.c */
 void FreeArgv(char **argv);
 char **ParseArgv(const char *s, int flag);
+char *BackSlashDecode(const char *s, const char *e);
+int BackSlash(const char *s, char *res);
 #define ARGV_COMMENT	(1 << 0)
 #define ARGV_COMMA	(1 << 1)
 
-/* from libvarnish/crc32.c */
-uint32_t crc32(uint32_t crc, const void *p1, unsigned l);
-uint32_t crc32_l(const void *p1, unsigned l);
-
 /* from libvarnish/num.c */
 const char *str2bytes(const char *p, uintmax_t *r, uintmax_t rel);
 
 /* from libvarnish/subproc.c */
 typedef void sub_func_f(void*);
-int SUB_run(struct vsb *sb, sub_func_f *func, void *priv, const char *name, int maxlines);
+int SUB_run(struct vsb *sb, sub_func_f *func, void *priv, const char *name,
+    int maxlines);
 
 /* from libvarnish/tcp.c */
 /* NI_MAXHOST and NI_MAXSERV are ridiculously long for numeric format */
 #define TCP_ADDRBUFSIZE		64
 #define TCP_PORTBUFSIZE		16
 
+#define TCP_Check(a) ((a) == 0 || errno == ECONNRESET || errno == ENOTCONN)
+
+#define TCP_Assert(a) assert(TCP_Check(a))
+
 void TCP_myname(int sock, char *abuf, unsigned alen, char *pbuf, unsigned plen);
+void TCP_hisname(int sock, char *abuf, unsigned alen, char *pbuf, unsigned plen);
 int TCP_filter_http(int sock);
-void TCP_blocking(int sock);
-void TCP_nonblocking(int sock);
+int TCP_blocking(int sock);
+int TCP_nonblocking(int sock);
+int TCP_linger(int sock, int linger);
 #ifdef SOL_SOCKET
 void TCP_name(const struct sockaddr *addr, unsigned l, char *abuf,
     unsigned alen, char *pbuf, unsigned plen);
@@ -75,6 +80,7 @@ void TCP_set_read_timeout(int s, double seconds);
 #endif
 
 /* from libvarnish/time.c */
+#define TIM_FORMAT_SIZE 30
 void TIM_format(double t, char *p);
 time_t TIM_parse(const char *p);
 double TIM_mono(void);
@@ -130,5 +136,18 @@ do {									\
 	lbv_assert(__func__, __FILE__, __LINE__, expl, errno, 3);	\
 	abort();							\
 } while (0)
+const char* svn_version(void);
 
-int inxorcise(int nochdir, int noclose);
+/* Safe printf into a fixed-size buffer */
+#define bprintf(buf, fmt, ...)						\
+	do {								\
+		assert(snprintf(buf, sizeof buf, fmt, __VA_ARGS__)	\
+		    < sizeof buf);					\
+	} while (0)
+
+/* Safe printf into a fixed-size buffer */
+#define vbprintf(buf, fmt, ap)						\
+	do {								\
+		assert(vsnprintf(buf, sizeof buf, fmt, ap)		\
+		    < sizeof buf);					\
+	} while (0)
