@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vrt.h 4428 2010-01-06 17:38:59Z tfheen $
+ * $Id: vrt.h 4703 2010-04-21 09:18:39Z tfheen $
  *
  * Runtime support for compiled VCL programs.
  *
@@ -136,8 +136,9 @@ const char *VRT_regsub(const struct sess *sp, int all, const char *,
     void *, const char *);
 
 void VRT_panic(struct sess *sp, const char *, ...);
-void VRT_purge(struct sess *sp, char *, ...);
-void VRT_purge_string(struct sess *sp, const char *, ...);
+void VRT_ban(struct sess *sp, char *, ...);
+void VRT_ban_string(struct sess *sp, const char *, ...);
+void VRT_purge(struct sess *sp, double ttl, double grace);
 
 void VRT_count(const struct sess *, unsigned);
 int VRT_rewrite(const char *, const char *);
