@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vss.h 2701 2008-06-16 21:25:36Z phk $
+ * $Id: vss.h 4588 2010-02-26 09:35:42Z phk $
  */
 
 /* vss.c */
@@ -34,5 +34,5 @@ int VSS_parse(const char *str, char **addr, char **port);
 int VSS_resolve(const char *addr, const char *port, struct vss_addr ***ta);
 int VSS_bind(const struct vss_addr *addr);
 int VSS_listen(const struct vss_addr *addr, int depth);
-int VSS_connect(const struct vss_addr *addr);
-int VSS_open(const char *str);
+int VSS_connect(const struct vss_addr *addr, int nonblock);
+int VSS_open(const char *str, double tmo);
