@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: binary_heap.c 3606 2009-02-05 11:02:44Z tfheen $
- *
  * Implementation of a binary heap API
  *
  * We use a malloc(3)/realloc(3) array to store the pointers using the
@@ -36,6 +34,9 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: binary_heap.c 4358 2009-11-19 19:03:25Z phk $")
+
 #include <unistd.h>
 #include <stdlib.h>
 
@@ -185,7 +186,7 @@ binheap_trickledown(const struct binheap *bh, unsigned u)
 		v2 = CHILD(u, 1);
 		if (v2 < bh->next && bh->cmp(bh->priv, A(bh, v2), A(bh, v1)))
 			v1 = v2;
-		if (bh->cmp(bh->priv, A(bh, u), A(bh, v1))) 
+		if (bh->cmp(bh->priv, A(bh, u), A(bh, v1)))
 			return;
 		binhead_swap(bh, u, v1);
 		u = v1;
@@ -425,7 +426,7 @@ main(int argc, char **argv)
 			binheap_delete(bh, ff[v].idx);
 		} else {
 			ff[v].key = random();
-			if (0) 
+			if (0)
 				printf("Insert [%u] %'u\n", v, ff[v].key);
 			else
 				printf("+%u", v);
