@@ -33,7 +33,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_action.c 4626 2010-03-17 09:38:47Z phk $")
+SVNID("$Id: vcc_action.c 4695 2010-04-19 13:03:14Z tfheen $")
 
 #include <stdio.h>
 #include <string.h>
@@ -101,7 +101,6 @@ parse_error(struct tokenlist *tl)
 		}
 	} else if (tl->t->tok == CNUM) {
 		Fb(tl, 1, "VRT_error(sp, %u", vcc_UintVal(tl));
-		vcc_NextToken(tl);
 	} else
 		Fb(tl, 1, "VRT_error(sp, 0");
 	if (tl->t->tok == CSTR) {
@@ -177,23 +176,8 @@ parse_set(struct tokenlist *tl)
 		case T_INCR:
 		case T_DECR:
 		case '=':
-			if (vp->fmt == TIME)
-				vcc_TimeVal(tl);
-			else if (vp->fmt == RTIME)
-				vcc_RTimeVal(tl);
-			else if (vp->fmt == SIZE)
-				vcc_SizeVal(tl);
-			else if (vp->fmt == FLOAT)
-				Fb(tl, 0, "%g", vcc_DoubleVal(tl));
-			else if (vp->fmt == INT) {
-				Fb(tl, 0, "%u", vcc_UintVal(tl));
-				vcc_NextToken(tl);
-			} else {
-				vsb_printf(tl->sb,
-				    "Cannot assign this variable type.\n");
-				vcc_ErrWhere(tl, vt);
-				return;
-			}
+			vcc_VarVal(tl, vp, vt);
+			ERRCHK(tl);
 			break;
 		default:
 			vsb_printf(tl->sb, "Invalid assignment operator.\n");
@@ -232,8 +216,7 @@ parse_set(struct tokenlist *tl)
 		Fb(tl, 0, ");\n");
 		break;
 	case HASH:
-		ExpectErr(tl, T_INCR);
-		vcc_NextToken(tl);
+		SkipToken(tl, T_INCR);
 		if (!vcc_StringVal(tl)) {
 			ERRCHK(tl);
 			vcc_ExpectedStringval(tl);
@@ -342,7 +325,7 @@ parse_purge(struct tokenlist *tl)
 	vcc_NextToken(tl);
 
 	if (tl->t->tok == VAR) {
-		Fb(tl, 1, "VRT_purge(sp,\n");
+		Fb(tl, 1, "VRT_ban(sp,\n");
 		tl->indent += INDENT;
 		while (1) {
 			ExpectErr(tl, VAR);
@@ -396,7 +379,7 @@ parse_purge(struct tokenlist *tl)
 		Fb(tl, 1, "0);\n");
 		tl->indent -= INDENT;
 	} else {
-		Fb(tl, 1, "VRT_purge_string(sp, ");
+		Fb(tl, 1, "VRT_ban_string(sp, ");
 		if (!vcc_StringVal(tl)) {
 			vcc_ExpectedStringval(tl);
 			return;
@@ -421,7 +404,7 @@ parse_purge_url(struct tokenlist *tl)
 	Expect(tl, '(');
 	vcc_NextToken(tl);
 
-	Fb(tl, 1, "VRT_purge(sp, \"req.url\", \"~\", ");
+	Fb(tl, 1, "VRT_ban(sp, \"req.url\", \"~\", ");
 	if (!vcc_StringVal(tl)) {
 		vcc_ExpectedStringval(tl);
 		return;
