@@ -51,7 +51,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_backend.c 4611 2010-03-16 08:52:37Z phk $")
+SVNID("$Id: vcc_backend.c 4681 2010-04-19 11:33:45Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -247,6 +247,7 @@ vcc_ParseProbe(struct tokenlist *tl)
 	struct token *t_did = NULL, *t_window = NULL, *t_threshold = NULL;
 	struct token *t_initial = NULL;
 	unsigned window, threshold, initial, status;
+	double t;
 
 	fs = vcc_FldSpec(tl,
 	    "?url",
@@ -259,8 +260,7 @@ vcc_ParseProbe(struct tokenlist *tl)
 	    "?initial",
 	    NULL);
 
-	ExpectErr(tl, '{');
-	vcc_NextToken(tl);
+	SkipToken(tl, '{');
 
 	window = 0;
 	threshold = 0;
@@ -293,23 +293,21 @@ vcc_ParseProbe(struct tokenlist *tl)
 			Fb(tl, 0, "\t\t\t\"\\r\\n\",\n");
 		} else if (vcc_IdIs(t_field, "timeout")) {
 			Fb(tl, 0, "\t\t.timeout = ");
-			vcc_TimeVal(tl);
+			vcc_TimeVal(tl, &t);
 			ERRCHK(tl);
-			Fb(tl, 0, ",\n");
+			Fb(tl, 0, "%g,\n", t);
 		} else if (vcc_IdIs(t_field, "interval")) {
 			Fb(tl, 0, "\t\t.interval = ");
-			vcc_TimeVal(tl);
+			vcc_TimeVal(tl, &t);
 			ERRCHK(tl);
-			Fb(tl, 0, ",\n");
+			Fb(tl, 0, "%g,\n", t);
 		} else if (vcc_IdIs(t_field, "window")) {
 			t_window = tl->t;
 			window = vcc_UintVal(tl);
-			vcc_NextToken(tl);
 			ERRCHK(tl);
 		} else if (vcc_IdIs(t_field, "initial")) {
 			t_initial = tl->t;
 			initial = vcc_UintVal(tl);
-			vcc_NextToken(tl);
 			ERRCHK(tl);
 		} else if (vcc_IdIs(t_field, "expected_response")) {
 			status = vcc_UintVal(tl);
@@ -320,12 +318,10 @@ vcc_ParseProbe(struct tokenlist *tl)
 				vcc_ErrWhere(tl, tl->t);
 				return;
 			}
-			vcc_NextToken(tl);
 			ERRCHK(tl);
 		} else if (vcc_IdIs(t_field, "threshold")) {
 			t_threshold = tl->t;
 			threshold = vcc_UintVal(tl);
-			vcc_NextToken(tl);
 			ERRCHK(tl);
 		} else {
 			vcc_ErrToken(tl, t_field);
@@ -334,8 +330,7 @@ vcc_ParseProbe(struct tokenlist *tl)
 			return;
 		}
 
-		ExpectErr(tl, ';');
-		vcc_NextToken(tl);
+		SkipToken(tl, ';');
 	}
 
 	if (t_threshold != NULL || t_window != NULL) {
@@ -376,8 +371,7 @@ vcc_ParseProbe(struct tokenlist *tl)
 	if (status > 0)
 		Fb(tl, 0, "\t\t.exp_status = %u,\n", status);
 	Fb(tl, 0, "\t},\n");
-	ExpectErr(tl, '}');
-	vcc_NextToken(tl);
+	SkipToken(tl, '}');
 }
 
 /*--------------------------------------------------------------------
@@ -399,6 +393,7 @@ vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 	struct fld_spec *fs;
 	struct vsb *vsb;
 	unsigned u;
+	double t;
 
 	Fh(tl, 1, "\n#define VGC_backend_%s %d\n", vgcname, tl->ndirector);
 
@@ -415,8 +410,7 @@ vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 	    NULL);
 	t_first = tl->t;
 
-	ExpectErr(tl, '{');
-	vcc_NextToken(tl);
+	SkipToken(tl, '{');
 
 	vsb = vsb_newauto();
 	AN(vsb);
@@ -451,49 +445,41 @@ vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 			assert(tl->t->dec != NULL);
 			t_host = tl->t;
 			vcc_NextToken(tl);
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "port")) {
 			ExpectErr(tl, CSTR);
 			assert(tl->t->dec != NULL);
 			t_port = tl->t;
 			vcc_NextToken(tl);
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "host_header")) {
 			ExpectErr(tl, CSTR);
 			assert(tl->t->dec != NULL);
 			t_hosthdr = tl->t;
 			vcc_NextToken(tl);
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "connect_timeout")) {
 			Fb(tl, 0, "\t.connect_timeout = ");
-			vcc_TimeVal(tl);
+			vcc_TimeVal(tl, &t);
 			ERRCHK(tl);
-			Fb(tl, 0, ",\n");
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			Fb(tl, 0, "%g,\n", t);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "first_byte_timeout")) {
 			Fb(tl, 0, "\t.first_byte_timeout = ");
-			vcc_TimeVal(tl);
+			vcc_TimeVal(tl, &t);
 			ERRCHK(tl);
-			Fb(tl, 0, ",\n");
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			Fb(tl, 0, "%g,\n", t);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "between_bytes_timeout")) {
 			Fb(tl, 0, "\t.between_bytes_timeout = ");
-			vcc_TimeVal(tl);
+			vcc_TimeVal(tl, &t);
 			ERRCHK(tl);
-			Fb(tl, 0, ",\n");
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			Fb(tl, 0, "%g,\n", t);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "max_connections")) {
 			u = vcc_UintVal(tl);
-			vcc_NextToken(tl);
 			ERRCHK(tl);
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 			Fb(tl, 0, "\t.max_connections = %u,\n", u);
 		} else if (vcc_IdIs(t_field, "saintmode_threshold")) {
 			u = vcc_UintVal(tl);
@@ -507,11 +493,9 @@ vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 				vsb_printf(tl->sb, " at\n");
 				vcc_ErrWhere(tl, tl->t);
 			}
-			vcc_NextToken(tl);
 			ERRCHK(tl);
 			saint = u;
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "probe")) {
 			vcc_ParseProbe(tl);
 			ERRCHK(tl);
@@ -616,8 +600,7 @@ vcc_ParseBackendHost(struct tokenlist *tl, int serial, char **nm)
 		}
 		vcc_AddRef(tl, h->name, R_BACKEND);
 		vcc_NextToken(tl);
-		ExpectErr(tl, ';');
-		vcc_NextToken(tl);
+		SkipToken(tl, ';');
 		*nm = h->vgcname;
 	} else if (tl->t->tok == '{') {
 		t = tl->t;
@@ -721,13 +704,10 @@ vcc_ParseDirector(struct tokenlist *tl)
 			vcc_ErrWhere(tl, tl->t_policy);
 			return;
 		}
-		ExpectErr(tl, '{');
-		vcc_NextToken(tl);
+		SkipToken(tl, '{');
 		dl->func(tl);
-		if (!tl->err) {
-			ExpectErr(tl, '}');
-			vcc_NextToken(tl);
-		}
+		if (!tl->err)
+			SkipToken(tl, '}');
 		Fi(tl, 0,
 		    "\tVRT_init_dir(cli, VCL_conf.director, \"%.*s\",\n",
 		    PF(tl->t_policy));
