@@ -30,7 +30,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_dir_random.c 4411 2009-12-16 11:41:36Z phk $")
+SVNID("$Id: vcc_dir_random.c 4770 2010-05-05 08:38:16Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -69,9 +69,7 @@ vcc_ParseRandomDirector(struct tokenlist *tl)
 			ExpectErr(tl, CNUM);
 			retries = vcc_UintVal(tl);
 			ERRCHK(tl);
-			vcc_NextToken(tl);
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 		} else {
 			ErrInternal(tl);
 		}
@@ -88,8 +86,7 @@ vcc_ParseRandomDirector(struct tokenlist *tl)
 		t_be = tl->t;
 		vcc_ResetFldSpec(mfs);
 
-		ExpectErr(tl, '{');
-		vcc_NextToken(tl);
+		SkipToken(tl, '{');
 		Fc(tl, 0, "\t{");
 
 		while (tl->t->tok != '}') {	/* Member fields */
@@ -115,9 +112,7 @@ vcc_ParseRandomDirector(struct tokenlist *tl)
 					return;
 				}
 				Fc(tl, 0, "%s .weight = %u", first, u);
-				vcc_NextToken(tl);
-				ExpectErr(tl, ';');
-				vcc_NextToken(tl);
+				SkipToken(tl, ';');
 			} else {
 				ErrInternal(tl);
 			}
